import random

import pytest
from rest_framework.test import APIClient

from conftest import assert_data
from django.contrib.auth.models import Permission, User
from program.models import MediaSource, Show
from program.tests.factories import (
    MediaFactory,
    MediaSourceFactory,
    UserWithPermissionsFactory,
)

pytestmark = pytest.mark.django_db


def url(media=None):
    if media:
        return f"/api/v1/media/{media.id}/"
    else:
        return "/api/v1/media/"


def media_data(owned_show):
    return {
        "description": "DESCRIPTION",
        "show_id": owned_show.id,
    }


def media_entries_data(show, file_id, entries=1):
    return [
        {
            "duration": random.uniform(1975, 2024),
            "file_id": file_id,
            "order": n + 1,
            "uri": f"file://{show.id}/{file_id}",
        }
        for n in range(entries)
    ]


@pytest.fixture
def user_with_media_perms() -> User:
    """User with add_media, change_media, delete_media permission"""

    permissions = Permission.objects.filter(
        codename__in=["add_media", "change_media", "delete_media"]
    )

    return UserWithPermissionsFactory.create(user_permissions=permissions)


@pytest.fixture
def api_client_media_perms(api_client, user_with_media_perms) -> APIClient:
    """Authenticated API client for user with {add,change,delete}_media permissions"""

    api_client.force_authenticate(user_with_media_perms)
    yield api_client
    api_client.force_authenticate()


@pytest.fixture
def owned_show_media_perms(user_with_media_perms, show) -> Show:
    """Show owned by user with media permissions"""

    show.owners.set([user_with_media_perms])
    show.save()

    return show


@pytest.fixture
def media_source(media) -> MediaSource:
    return MediaSourceFactory(media=media)


@pytest.fixture
def owned_show(common_user1, show) -> Show:
    """Show owned by a common user"""

    show.owners.set([common_user1])
    show.save()

    return show


def test_create_media(user_with_media_perms, api_client_media_perms, owned_show_media_perms):
    data = media_data(owned_show_media_perms)

    response = api_client_media_perms.post(url(), data=data)

    assert response.status_code == 201

    assert_data(response, data)


def test_retrieve_media(user_with_media_perms, api_client_media_perms, show, media):
    response = api_client_media_perms.get(url(media=media))

    assert response.status_code == 200


def test_list_media(api_client_media_perms, show):
    MEDIA = 3
    MediaFactory.create_batch(size=MEDIA, show=show)

    response = api_client_media_perms.get(url())

    assert response.status_code == 200
    assert len(response.data) == MEDIA


def test_delete_media(api_client_media_perms, owned_show_media_perms, media):
    response = api_client_media_perms.delete(url(media=media))

    print(response.data)

    assert response.status_code == 204


def test_update_description(api_client_media_perms, owned_show_media_perms, media):
    update = {"description": "DESCRIPTION"}

    response = api_client_media_perms.patch(url(media=media), data=update)

    assert response.status_code == 200
    assert response.data["description"] == update["description"]


def test_update_playout_mode(api_client_media_perms, owned_show_media_perms, media):
    update = {"playout_mode": "LINEAR"}

    response = api_client_media_perms.patch(url(media=media), data=update, format="json")
    print(response.data)
    assert response.status_code == 200
    assert response.data["playout_mode"] == update["playout_mode"]


def test_update_entries(api_client_media_perms, owned_show_media_perms, media):
    media_entries = media_entries_data(owned_show_media_perms, 42, 3)

    update = {"entries": media_entries}

    response = api_client_media_perms.patch(url(media=media), data=update, format="json")

    assert response.status_code == 200

    for entry, update in zip(response.data["entries"], update["entries"]):
        assert entry["duration"] == update["duration"]
        assert entry["file_id"] == update["file_id"]
        assert entry["uri"] == update["uri"]


def test_clear_entries(api_client_media_perms, owned_show_media_perms, media, media_source):
    update = {"entries": []}

    response = api_client_media_perms.patch(url(media=media), data=update, format="json")

    assert response.status_code == 200
    assert len(response.data["entries"]) == 0
