import json
import os
import sys
from pathlib import Path
from typing import Literal, cast

import dj_database_url
import ldap
from corsheaders.defaults import default_headers
from django_auth_ldap.config import LDAPSearch, PosixGroupType

BASE_DIR = Path(__file__).resolve(strict=True).parent.parent

LOCALE_PATHS = (BASE_DIR / "locale",)

MEDIA_ROOT = BASE_DIR / "site_media"
MEDIA_URL = "/site_media/"

STATIC_ROOT = BASE_DIR / "static"
STATIC_URL = "/static/"

ROOT_URLCONF = "steering.urls"

DEBUG = True if os.getenv("DEBUG") else False
SITE_ID = 1

# Must be set if DEBUG is False
ALLOWED_HOSTS = os.getenv("ALLOWED_HOSTS", default="127.0.0.1,localhost").split(",")

CORS_ALLOWED_ORIGINS = os.getenv(
    "CORS_ALLOWED_ORIGINS",
    default="http://localhost:8080,http://localhost:8040",
).split(",")
CORS_ALLOW_CREDENTIALS = True
CORS_ALLOW_HEADERS = list(default_headers) + [
    "content-disposition",
]

DATABASES = {"default": dj_database_url.config(default=f"sqlite:///{BASE_DIR}/db.sqlite3")}

TIME_ZONE = os.getenv("TZ", default="Europe/Vienna")
LANGUAGE_CODE = os.getenv("LANGUAGE_CODE", default="en-us")

USE_TZ = True  # django-oidc-provider needs timezones in database

SECRET_KEY = os.getenv("SECRET_KEY", default="secret-key")

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.request",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

MIDDLEWARE = (
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "djangorestframework_camel_case.middleware.CamelCaseMiddleWare",
)

REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly"
    ],
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "program.auth.OidcOauth2Auth",
    ],
    "DEFAULT_FILTER_BACKENDS": ["django_filters.rest_framework.DjangoFilterBackend"],
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_PARSER_CLASSES": (
        "djangorestframework_camel_case.parser.CamelCaseFormParser",
        "djangorestframework_camel_case.parser.CamelCaseMultiPartParser",
        "djangorestframework_camel_case.parser.CamelCaseJSONParser",
    ),
    "DEFAULT_RENDERER_CLASSES": (
        "djangorestframework_camel_case.render.CamelCaseJSONRenderer",
        "djangorestframework_camel_case.render.CamelCaseBrowsableAPIRenderer",
    ),
    "EXCEPTION_HANDLER": "steering.views.full_details_exception_handler",
}

SPECTACULAR_SETTINGS = {
    "TITLE": "AURA Steering API",
    "DESCRIPTION": "Programme/schedule management for Aura",
    "POSTPROCESSING_HOOKS": [
        "drf_spectacular.hooks.postprocess_schema_enums",
        "steering.schema.add_enum_documentation",
        "steering.schema.fix_schedule_pk_type",
        "drf_spectacular.contrib.djangorestframework_camel_case.camelize_serializer_fields",
    ],
    "VERSION": "1.0.0",
}

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.admin",
    "django.contrib.staticfiles",
    "program",
    "versatileimagefield",
    "rest_framework",
    "django_filters",
    "django_extensions",
    "drf_spectacular",
    "oidc_provider",
    "corsheaders",
    "django_json_widget",
)

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

# When generating schedules/timeslots:
# If last_date wasn't set manually, add timeslots until these number of days after the start_date
AUTO_SET_LAST_DATE_TO_DAYS_IN_FUTURE = 365

# If last_date wasn't set manually, add timeslots until the end of the current year
# Overrides the above setting if True
AUTO_SET_LAST_DATE_TO_END_OF_YEAR = True

# URL to CBA - Cultural Broadcasting Archive
CBA_URL = "https://cba.media"

# Contact cba@fro.at to get whitelisted and get an API KEY
# Leave empty to disable requests to CBA
CBA_API_KEY = os.getenv("CBA_API_KEY", default="")

# URL to CBA ajax handler (used for retrieving additional data or increasing stream counter)
CBA_AJAX_URL = CBA_URL + "/wp-admin/admin-ajax.php"

# URL to CBA's REST API with trailing slash
CBA_REST_API_URL = CBA_URL + "/wp-json/wp/v2/"

# OIDC Provider Settings
LOGIN_URL = "/admin/login/"  # Login page OIDC redirects to
OIDC_EXTRA_SCOPE_CLAIMS = "steering.oidc_provider_settings.AuraScopeClaims"

# OIDC Provider extra scope claims
# - Superusers and members of this group are privileged.
PRIVILEGED_GROUP = os.getenv("PRIVILEGED_GROUP", default="Programme Manager")
# - Members of these groups are entitled.
ENTITLED_GROUPS = os.getenv("ENTITLED_GROUPS", default="Host,Host+").split(",")

# The API will filter the active shows using one of these filters
# One of these should to be True.
# - if both are True, shows will be filtered using the field AND the annotation
# - if both are False, shows will NOT be filtered
FILTER_ACTIVE_SHOWS_USING = {
    # shows will be filtered using this field
    "is_active": os.getenv("FILTER_ACTIVE_SHOWS_USING_FIELD", default=False),
    # shows will be filtered using this annotation
    "max_timeslot_start": os.getenv("FILTER_ACTIVE_SHOWS_USING_ANNOTATION", default=True),
}

# WSGI_APPLICATION = 'steering.wsgi.application';

if os.getenv("USE_LDAP_AUTH"):
    # we don't have defaults for these because they are set in docker-compose
    AUTHENTICATION_BACKENDS = (
        "django_auth_ldap.backend.LDAPBackend",
        "django.contrib.auth.backends.ModelBackend",
    )

    AUTH_LDAP_ALWAYS_UPDATE_USER = os.getenv("AUTH_LDAP_ALWAYS_UPDATE_USER")
    AUTH_LDAP_BIND_DN = os.getenv("AUTH_LDAP_BIND_DN")
    AUTH_LDAP_BIND_PASSWORD = os.getenv("AUTH_LDAP_BIND_PASSWORD")
    AUTH_LDAP_FIND_GROUP_PERMS = os.getenv("AUTH_LDAP_FIND_GROUP_PERMS")
    AUTH_LDAP_GROUP_SEARCH = LDAPSearch(
        base_dn=os.getenv("AUTH_LDAP_GROUP_SEARCH_BASE_DN"),
        scope=ldap.SCOPE_SUBTREE,
        filterstr=os.getenv("AUTH_LDAP_GROUP_SEARCH_FILTERSTR"),
    )
    AUTH_LDAP_GROUP_TYPE = PosixGroupType()
    AUTH_LDAP_MIRROR_GROUPS = os.getenv("AUTH_LDAP_MIRROR_GROUPS")
    AUTH_LDAP_SERVER_URI = os.getenv("AUTH_LDAP_SERVER_URI")
    AUTH_LDAP_USER_ATTR_MAP = {
        "first_name": os.getenv("AUTH_LDAP_USER_ATTR_MAP_FIRST_NAME"),
        "last_name": os.getenv("AUTH_LDAP_USER_ATTR_MAP_LAST_NAME"),
        "email": os.getenv("AUTH_LDAP_USER_ATTR_MAP_EMAIL"),
    }
    AUTH_LDAP_USER_DN_TEMPLATE = os.getenv("AUTH_LDAP_USER_DN_TEMPLATE")
    AUTH_LDAP_USER_FLAGS_BY_GROUP = {
        "is_active": os.getenv("AUTH_LDAP_USER_FLAGS_BY_GROUP_ACTIVE"),
        "is_staff": os.getenv("AUTH_LDAP_USER_FLAGS_BY_GROUP_STAFF"),
        "is_superuser": os.getenv("AUTH_LDAP_USER_FLAGS_BY_GROUP_SUPERUSER"),
    }

PORT = os.getenv("STEERING_PORT")
AURA_PROTO = os.getenv("AURA_PROTO", default="http")
AURA_HOST = os.getenv("AURA_HOST", default="localhost")

# SITE_URL is used by django-oidc-provider and openid-configuration will break if not set correctly
SITE_URL = os.getenv(
    "SITE_URL",
    default=f"{AURA_PROTO}://{AURA_HOST}:{PORT}" if PORT else f"{AURA_PROTO}://{AURA_HOST}",
)
# SITE_URL and SITE_WEBSITE_URL are likely the same (hence the default),
# but may differ in some setups that serve steering on a separate domain
# instead of the default sub-path.
# This will usually point to the URL of the dashboard.
SITE_WEBSITE_URL = os.getenv("SITE_WEBSITE_URL", default=SITE_URL)
SITE_NAME = os.getenv("SITE_NAME", default="AURA Backend")

if AURA_PROTO == "https":
    CSRF_TRUSTED_ORIGINS = [f"{AURA_PROTO}://{AURA_HOST}"]

OPERATION_MODE_TYPE = Literal["default", "tests"]
OPERATION_MODE: OPERATION_MODE_TYPE
OPERATION_MODE = cast(OPERATION_MODE_TYPE, os.getenv("OPERATION_MODE", default="default"))

# TODO: find a better way. This is limited to settings that can be JSON-encoded.
if "LOGGING_JSON" in os.environ:
    LOGGING = json.loads(os.getenv("LOGGING_JSON"))
else:
    LOGGING = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "django.server": {
                "()": "django.utils.log.ServerFormatter",
                "format": "[{server_time}] {message}",
                "style": "{",
            }
        },
        "handlers": {
            "file": {
                "class": "logging.FileHandler",
                "filename": os.path.abspath(os.path.join(BASE_DIR, "logs", "steering.log")),
                "formatter": "django.server",
            },
        },
        "loggers": {
            "django": {
                "handlers": ["file"],
                "level": os.getenv("STEERING_LOG_LEVEL", "INFO"),
                "propagate": True,
            },
        },
    }

# ATTENTION:
# Don’t add any configuration settings after this, so that administrators can override them.
try:
    sys.path.insert(0, "/etc/steering")
    from steering_settings import *  # noqa: F401,F403
except ImportError:
    if os.path.exists("/etc/steering/steering_settings.py"):
        raise
